@@ -18,16 +18,10 @@ char *       lk_playlist_escape          (char * data, unsigned int length);
 unsigned int lk_playlist_unescape_inplace(char * str);
 unsigned int lk_playlist_unescape        (char * str, char ** data);
 
-playlist *   lk_playlist_create          (char * name);
-playlist *   lk_playlist_read            (char * name);
-int          lk_playlist_clear           (playlist * pl);
 int          lk_playlist_delete          (playlist * pl, int karma);
-int          lk_playlist_append          (playlist * pl, uint32_t fid,
-                                          uint32_t fid_generation);
 int          lk_playlist_remove          (playlist * pl, unsigned int n);
 int          lk_playlist_insert          (playlist * pl, unsigned int n,
                                           uint32_t fid,
                                           uint32_t fid_generation);
-int          lk_playlist_write           (playlist * pl, int karma);
 
 #endif /* _PLAYLIST_H */
