@@ -344,7 +344,7 @@ int lk_properties_save(void)
         return -1;
     }
 
-    for(i=0,j=16; i<props.count-1; j++) {
+    for(i=0,j=16; i<props.count; j++) {
         if(props.fidTable[j]){
             buf=lk_properties_export(j*16);
             write(fd, buf, strlen(buf));
@@ -377,7 +377,7 @@ void lk_properties_inc_devgeneration(void)
 }
 
 
-int lk_properties_cache_obsolete(char *device_settings)
+int lk_properties_cache_obsolete(void)
 {
     char *fname;
     struct stat modified;
@@ -424,7 +424,7 @@ uint32_t *lk_properties_andOrSearch(int mode, uint32_t *search_in, char *key,
         if(and)                           /* AND merge with zero elements */
             return NULL;
         hash=hash_string((const unsigned char *)key, HASHSIZE);
-        while(count < props.count){
+        while(count <= props.count){
             if(props.fidTable[fid]){
                 ++count;
                 val = (char *)hash_find_hash(props.fidTable[fid], hash, key);
@@ -455,7 +455,7 @@ uint32_t *lk_properties_andOrSearch(int mode, uint32_t *search_in, char *key,
         memcpy(result, search_in, (i) * sizeof(uint32_t));
         
         hash=hash_string((const unsigned char *)key, HASHSIZE);
-        while(count < props.count){
+        while(count <= props.count){
             if(props.fidTable[fid]){
                 ++count;
                 val=(char *)hash_find_hash(props.fidTable[fid], hash, key);
@@ -517,7 +517,7 @@ int lk_properties_write_property(FILE *fp, char *attr, smalldb_type_t type,
     offset_16 = (uint16_t *) offset;
 
     if (type == SDB_BLOB || type == SDB_STRING) {
-        for(i=0,j=16,off=0; i<props.count-1; j++) {
+        for(i=0,j=16,off=0; i<props.count; j++) {
             if(!props.fidTable[j]) continue;
             str = lk_properties_get_property(j*16,attr);
             if (str) {
@@ -612,7 +612,7 @@ int lk_properties_write_property(FILE *fp, char *attr, smalldb_type_t type,
             }
         }
     } else if (type == SDB_LE32) {
-        for(i=0,j=16; i<props.count-1; j++) {
+        for(i=0,j=16; i<props.count; j++) {
             if(!props.fidTable[j]) continue;
             str = lk_properties_get_property(j*16,attr);
             if(!str)
@@ -623,7 +623,7 @@ int lk_properties_write_property(FILE *fp, char *attr, smalldb_type_t type,
         }
         fwrite(offset, i*sizeof(uint32_t), 1, fp);
     } else if (type == SDB_LE16) {
-        for(i=0,j=16; i<props.count-1; j++) {
+        for(i=0,j=16; i<props.count; j++) {
             if(!props.fidTable[j]) continue;
             str = lk_properties_get_property(j*16,attr);
             if(!str)
@@ -634,7 +634,7 @@ int lk_properties_write_property(FILE *fp, char *attr, smalldb_type_t type,
         }
         fwrite(offset_16, i*sizeof(uint16_t), 1, fp);
     } else if (type == SDB_CHAR) {
-        for(i=0,j=16; i<props.count-1; j++) {
+        for(i=0,j=16; i<props.count; j++) {
             if(!props.fidTable[j]) continue;
             str = lk_properties_get_property(j*16,attr);
             if(!str)
@@ -695,7 +695,7 @@ static int smalldb_init(karma_db_t *db)
 
 int lk_properties_write_smalldb(char *usbMountPoint, karma_db_t *db)
 {
-    uint32_t i, len, arrsz, nfiles = props.count-1;
+    uint32_t i, len, arrsz, nfiles = props.count;
     uint32_t ret = 0;
     uint32_t *offset = NULL;
     table_t *table = NULL;
@@ -725,6 +725,10 @@ int lk_properties_write_smalldb(char *usbMountPoint, karma_db_t *db)
 
     sprintf(tmp, "%s%s", usbMountPoint, RK_SMALLDB);
     fp = fopen(tmp, "w+");
+    if (!fp) {
+        mk_path(tmp);
+        fp = fopen(tmp, "w+");
+    }
     if (!fp) {
         perror(tmp);
         goto err;
@@ -738,9 +742,11 @@ int lk_properties_write_smalldb(char *usbMountPoint, karma_db_t *db)
         goto err;
 
     if (db->num_attribs == 0)
+    {
         ret = smalldb_init(db);
-    if (ret)
-        goto err;
+        if (ret)
+            goto err;
+    }
 
     db->ctime = (int) time(NULL);
     fwrite(&db->version, sizeof(uint32_t), 1, fp);
@@ -793,18 +799,15 @@ int lk_properties_write_smalldb(char *usbMountPoint, karma_db_t *db)
 
     fwrite(&db->ctime, sizeof(uint32_t), 1, fp);
     fclose(fp);
-
-    if (arr) free(arr);
-    if (offset) free(offset);
-    if (table) free(table);
-
-    return 0;
+    goto out;
 
 err:
+    ret = 1;
+out:
     if (arr) free(arr);
     if (offset) free(offset);
     if (table) free(table);
     if (tmp) free(tmp);
 
-    return 1;
+    return ret;
 }
