@@ -41,13 +41,10 @@ enum { TYPE_DATA, TYPE_MP3, TYPE_OGG, TYPE_FLAC, TYPE_WAV, TYPE_WMA };
 
 static int write_dupes = 0;
 
-static void add_general(HASH * p, const char *filename);
+static void add_general(HASH * p, const char *filename, int updateLength);
 static int get_ogg_props(HASH *p, const char *filename);
 static int get_flac_props(HASH *p, const char *filename);
-/* *********************** EV *********************** */
-/* static int get_mp3_props(HASH *p, mp3info *mp3); */
-static int get_mp3_props(HASH *p, mp3info *mp3, uint64_t length);
-/* ************************************************** */
+static int get_mp3_props(HASH *p, mp3info *mp3);
 static int get_taxi_props(HASH *p);
 static int get_wave_props(HASH *p, const char *filename);
 static void add_tag(HASH *p, char *key, char *value);
@@ -129,8 +126,8 @@ int lk_rio_read(int rio, uint32_t fid, const char * filename)
     return 0;
 }
 
-static int lk_rio_do_update_props_from_tags(int rio, uint32_t fid,
-                                            const char *filename)
+static int lk_rio_do_update_props_from_tags(
+           int rio, uint32_t fid, const char *filename, int updateLength)
 {
     int got=-1;
     struct stat len;
@@ -160,14 +157,7 @@ static int lk_rio_do_update_props_from_tags(int rio, uint32_t fid,
         return -1;
     }
 
-    /* *********************** EV *********************** */
-    /* to fix bad length & duration yield by lk_rio_do_update_props_from_tags */
-    tmp = lk_properties_get_property(fid, "length");
-    if(tmp != NULL)
-      length = strtoull(tmp, NULL, 10);
-    /* ************************************************** */
-
-    add_general(props, filename);
+    add_general(props, filename, updateLength);
     lk_properties_set_property_hash(props, "rid", rid);
 
     if (type == TYPE_OGG)
@@ -175,16 +165,13 @@ static int lk_rio_do_update_props_from_tags(int rio, uint32_t fid,
     else if (type == TYPE_FLAC)
         got=get_flac_props(props, filename);
     else if (type == TYPE_MP3)
-        /* *********************** EV *********************** */
-        /* got=get_mp3_props(props, &mp3); */
-        got=get_mp3_props(props, &mp3, length);
-        /* ************************************************** */
+        got=get_mp3_props(props, &mp3);
     else if (type == TYPE_WAV)
         got=get_wave_props(props, filename);
     /*
     else if (type == TYPE_WMA)
         got=get_wma_props(props, filename);
-     */    
+    */    
     else
         got=get_taxi_props(props);
 
@@ -198,7 +185,7 @@ static int lk_rio_do_update_props_from_tags(int rio, uint32_t fid,
 static char *lk_rio_create_temp_file(int rio, uint32_t fid)
 {
 #define RID_BLOCKS 65536
-    int fd, ret;
+    int fd, ret, type;
     int64_t len;
     uint64_t blk, got, off, length;
     char *filename = NULL, *tmp;
@@ -214,7 +201,7 @@ static char *lk_rio_create_temp_file(int rio, uint32_t fid)
     blk = RID_BLOCKS + 4096;
     off = 0;
     ret = lk_karma_read_file_chunk(rio, off, blk, fid, &tmp, &got);
-    if (ret != 0 || got != blk)
+    if (ret != 0) /* || got != blk) */        /* may be < blk for taxy files */
         goto err;
 
     len = strlen(karma_tmpdir) + strlen("/lkarmaXXXXXX");
@@ -232,11 +219,11 @@ static char *lk_rio_create_temp_file(int rio, uint32_t fid)
 
     memset(&mp3, 0, sizeof(mp3));
     mp3.filename = filename;
-    get_file_type(&mp3);
-
-    fd = open(filename, O_WRONLY);
+    type = get_file_type(&mp3);
+    if(type == TYPE_DATA) return filename;    /* that's enough for taxi files */
 
     /* read the rest of the header if 4k wasnt enough */
+    fd = open(filename, O_WRONLY);
     len = mp3.offset + RID_BLOCKS - blk;
     if (len > 0) {
         off = blk;
@@ -244,6 +231,7 @@ static char *lk_rio_create_temp_file(int rio, uint32_t fid)
         ret = lk_karma_read_file_chunk(rio, off, blk, fid, &tmp, &got);
         if (ret != 0 || got != blk)
             goto err;
+
         ret = lseek(fd, off, SEEK_SET);
         ret = write(fd, tmp, got);
         free(tmp);
@@ -270,6 +258,7 @@ static char *lk_rio_create_temp_file(int rio, uint32_t fid)
     ret = lk_karma_read_file_chunk(rio, off, blk, fid, &tmp, &got);
     if (ret != 0 || got != blk)
         goto err;
+
     ret = lseek(fd, (mp3.offset + RID_BLOCKS), SEEK_SET);
     ret = write(fd, tmp, got);
     free(tmp);
@@ -304,7 +293,11 @@ int lk_rio_update_props_from_tags(int rio, uint32_t fid)
     if (!filename)
         return -1;
 
-    ret = lk_rio_do_update_props_from_tags(rio, fid, filename);
+    /* Don't update the length propery (last arg = 0).  Length of fid
+     * was already correctly set when the fid was first created and
+     * length of files provided by lk_rio_create_temp_file() are wrong
+     */
+    ret = lk_rio_do_update_props_from_tags(rio, fid, filename, 0);
 
     if (using_usb == 0 || ret)
         unlink(filename);
@@ -348,7 +341,7 @@ static uint32_t lk_rio_do_write(int fdb, int rio, const char * filename)
     if (!fid)
         fid = lk_properties_new_property(); 
 
-    got = lk_rio_do_update_props_from_tags(rio, fid, filename);
+    got = lk_rio_do_update_props_from_tags(rio, fid, filename, 1);
     if (got != 0) {
         lk_properties_del_property(fid);
         close(fd);
@@ -407,21 +400,24 @@ uint32_t lk_rio_write_fdb(int rio, const char * filename)
     return lk_rio_do_write(1, rio, filename);
 }
 
-static void add_general(HASH * p, const char *filename)
+static void add_general(HASH * p, const char *filename, int updateLength)
 {
     time_t generation_time;
     char *t;
     struct stat len;
     
-    lstat(filename, &len);
-    
+    if(updateLength)
+        lstat(filename, &len);
+
     generation_time=time(NULL);
+
     lk_properties_set_property_hash(p, "profile", "");
     lk_properties_set_property_hash(p, "fid_generation", 
                                     simple_itoa(generation_time));
     lk_properties_set_property_hash(p, "ctime", 
                                     simple_itoa(generation_time));
-    lk_properties_set_property_hash(p, "length", simple_itoa(len.st_size));
+    if(updateLength)
+        lk_properties_set_property_hash(p, "length", simple_itoa(len.st_size));
     t=strrchr(filename, '/');
     t=utf8_to_codeset((char *)(t?t+1:filename));
     lk_properties_set_property_hash(p, "title", t);
@@ -458,14 +454,13 @@ static int get_flac_props(HASH *p, const char *filename)
     return set_tag_props(p, filename, TagLib_File_FLAC);
 }
 
-/* *********************** EV *********************** */
-/* static int get_mp3_props(HASH *p, mp3info *mp3) */
-static int get_mp3_props(HASH *p, mp3info *mp3, uint64_t length)
-/* ************************************************** */
+static int get_mp3_props(HASH *p, mp3info *mp3)
 {
     int ret = 0, brate = 0;
     char bitrate[5];
-    
+    char* aux;
+    uint64_t length;
+
     lk_properties_set_property_hash(p, "type", "tune");
     lk_properties_set_property_hash(p, "codec", "mp3");
     lk_properties_set_property_hash(p, "offset", simple_itoa(mp3->offset));
@@ -478,21 +473,18 @@ static int get_mp3_props(HASH *p, mp3info *mp3, uint64_t length)
     ret = set_tag_props(p, mp3->filename, TagLib_File_MPEG);
     
     if(!mp3->vbr){
-        /* *********************** EV *********************** */
         brate = header_bitrate(&mp3->header);
         strcpy(bitrate, "fs");
         strcat(bitrate, simple_itoa(brate));
         lk_properties_set_property_hash(p, "bitrate", bitrate);
-        /* fixing bad length & duration yield
-           by lk_rio_do_update_props_from_tags */
-	if(length>0) {
-            lk_properties_set_property_hash(p, "length", simple_itoa(length));
+        /* fix bad duration yield by lk_rio_do_update_props_from_tags */
+        aux = lk_properties_get_property_hash(p, "length");
+        if(aux != NULL) {
+            length = strtoull(aux, NULL, 10);
             lk_properties_set_property_hash(p, "duration", 
-                                               simple_itoa(8*(length/brate)));
+                                                simple_itoa(8*(length/brate)));
 	}
-        /* ************************************************** */
     }
-    
     return ret;
 }
 
